/*
    Copyright (c) 2004  Joey Schulze <joey@infodrom.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <strings.h>

/* #define FROM_DETECTION */

struct mail {
  char from[128];
  char subject[128];
  int priority;
  off_t size;
};

#define HDR_LEN	128

/*
 * Like strncpy() but with terminated result.
 */
char *stringcopy(char *dest, const char *src, size_t n)
{
  strncpy(dest, src, n-1);
  dest[n-1] = '\0';
  return dest;
}

int inspect_mbox(char *path, char *prefix, off_t size, int opt_flags)
{
  FILE *f;
  char buf[HDR_LEN];
  char tmp[512];
  char *cp;
  int inheader = 1;
  int newmail = 0;

  char from_[HDR_LEN] = "";
  char from[HDR_LEN] = "";
#ifdef FROM_DETECTION
  char to[HDR_LEN] = "";
#endif
  char subject[HDR_LEN] = "";
  int priority = 0;

  if ((f = fopen(path, "r")) == NULL)
    return 0;

  if (size > 0 && fseek(f, size, SEEK_SET) != 0)
    return 0;

  while (!feof(f)) {
    if ((cp = fgets(buf, sizeof(buf), f)) == NULL)
      continue;
    if (strlen(buf) > 0 && buf[strlen(buf)-1] == '\n') {
      buf[strlen(buf)-1] = '\0';
    } else {
      while (!feof(f) && fgets(tmp, sizeof(tmp), f) != NULL) {
	if (strlen(tmp) > 0 && tmp[strlen(tmp)-1] == '\n')
	  break;
      }
    }

    if (inheader) {
      if (strlen(buf) == 0) {
	inheader = 0;
	newmail = 1;

	/* TODO: display header */

#ifdef FROM_DETECTION
	from_[0] = from[0] = to[0] = subject[0] = '\0';
#else
	from_[0] = from[0] = subject[0] = '\0';
#endif
	priority = 0;
      } else {
	if (strncasecmp(buf, "From ", 5) == 0)
	  stringcopy(from_, buf+5, sizeof(from_));
	else if (strncasecmp(buf, "From: ", 6) == 0)
	  stringcopy(from, buf+6, sizeof(from));
#ifdef FROM_DETECTION
	else if (strncasecmp(buf, "To: ", 4) == 0)
	  stringcopy(to, buf+4, sizeof(to));
#endif
	else if (strncasecmp(buf, "Subject: ", 9) == 0)
	  stringcopy(subject, buf+9, sizeof(subject));
	else if (strncasecmp(buf, "Priority: ", 10) == 0)
	  priority = 1;
      }
    } else if (strncasecmp(buf, "From ", 5) == 0) {
      inheader = 1;
      stringcopy(from_, buf+5, sizeof(from_));
    }

  }

  return newmail;
}

int watch_mbox(char *path, char *prefix, off_t *size, int opt_flags)
{
  struct stat st;
  int newmail = 0;

  if (stat(path, &st) == 0) {
    if (st.st_size > *size)
      if (access(path, R_OK) == 0) {
	newmail = inspect_mbox(path, prefix, *size, opt_flags);
      }
    *size = st.st_size;
    return newmail;
  } else {
    *size = 0;
  }

  return 0;
}
